// generated by Fast Light User Interface Designer (fluid) version 1.0107

#include "OscilGenUI.h"
//Copyright (c) 2002-2005 Nasca Octavian Paul
//License: GNU GPL version 2
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

OscilSpectrum::OscilSpectrum(int x,int y, int w, int h, const char *label):Fl_Box(x,y,w,h,label) {
  oscil=NULL;
}

void OscilSpectrum::init(OscilGen *oscil_,int oscbase_,Master *master_) {
  oscil=oscil_;
oscbase=oscbase_;
master=master_;
}

void OscilSpectrum::draw() {
  int ox=x(),oy=y(),lx=w(),ly=h(),i;
const int maxdb=60;//must be multiple of 10
int GX=2;
int n=lx/GX-1;
if (n>OSCIL_SIZE/2) n=OSCIL_SIZE/2;

REALTYPE x;
REALTYPE spc[n];
for (i=0;i<n;i++) spc[i]=0.0;

pthread_mutex_lock(&master->mutex);
if (oscbase==0) oscil->getspectrum(n,spc,0);
    else oscil->getspectrum(n,spc,1);
pthread_mutex_unlock(&master->mutex);

//normalize
REALTYPE max=0;
for (i=0;i<n;i++){
   x=fabs(spc[i]);
   if (max<x) max=x;
}
if (max<0.000001) max=1.0;
max=max*1.05;

//draw

if (this->active_r()) fl_color(this->parent()->selection_color());
    else fl_color(this->parent()->color());
fl_line_style(FL_DOT);

for (i=1;i<maxdb/10;i++){
  int ky=(int)((REALTYPE)i*ly*10.0/maxdb)/2;
  ky*=2;
  fl_line(ox,oy+ky-1,ox+lx-2,oy+ky-1);
};

for (i=2;i<n;i++){
    int tmp=i*GX-2;
    if (i%10==1) fl_line_style(0);
       else fl_line_style(FL_DOT);
    fl_line(ox+tmp,oy+2,ox+tmp,oy+ly-2);
}

if (this->active_r()) fl_color(this->parent()->labelcolor());
    else fl_color(this->parent()->color());
fl_line_style(0);

//draws the spectrum
for (i=0;i<n;i++){
   int tmp=i*GX+2;
   x=spc[i]/max;
   
   if (x>dB2rap(-maxdb)) x=rap2dB(x)/maxdb+1;
       else x=0;

   int val=(int) ((ly-2)*x);
   if (val>0) fl_line(ox+tmp,oy+ly-2-val,ox+tmp,oy+ly-2);
};
}

PSlider::PSlider(int x,int y, int w, int h, const char *label):Fl_Slider(x,y,w,h,label) {
}

int PSlider::handle(int event) {
  int X=x(),Y=y(),W=w(),H=h();

if ((!Fl::event_buttons())|| (event==0)||(Fl::event_shift()==0)) return(Fl_Slider::handle(event));

if (!Fl::event_inside(X,Y,W,H)) {
	if (event==FL_DRAG){
		Fl_Slider::handle(FL_RELEASE);
		Fl_Slider::handle(FL_LEAVE);
		deactivate();
		activate();
		return(1);	
	}else{
		return(Fl_Slider::handle(event));
	};
} else {
	//Fl_Slider::handle(FL_FOCUS);
	Fl_Slider::handle(FL_PUSH);
};

return(1);
}

Oscilloscope::Oscilloscope(int x,int y, int w, int h, const char *label):Fl_Box(x,y,w,h,label) {
  oscil=NULL;
phase=64;
oscbase=0;
}

void Oscilloscope::init(OscilGen *oscil_,Master *master_) {
  oscil=oscil_;
master=master_;
}

void Oscilloscope::init(OscilGen *oscil_,int oscbase_,Master *master_) {
  oscil=oscil_;
oscbase=oscbase_;
master=master_;
}

void Oscilloscope::init(OscilGen *oscil_,int oscbase_,int phase_,Master *master_) {
  oscil=oscil_;
oscbase=oscbase_;
phase=phase_;
master=master_;
}

void Oscilloscope::draw() {
  int ox=x(),oy=y(),lx=w(),ly=h()-1,i;
REALTYPE smps[OSCIL_SIZE];
pthread_mutex_lock(&master->mutex);
if (oscbase==0) oscil->get(smps,-1.0);
    else oscil->getcurrentbasefunction(smps);
pthread_mutex_unlock(&master->mutex);

if (damage()!=1){
 fl_color(0,0,0);
 fl_rectf(ox,oy,lx,ly);
};

//normalize
REALTYPE max=0;
for (i=0;i<OSCIL_SIZE;i++)
   if (max<fabs(smps[i])) max=fabs(smps[i]);
//fprintf(stderr,"%.4f\n",max);
if (max<0.00001) max=1.0;
max=-max*1.05;

//draw
fl_line_style(FL_DASH);
if (this->active_r()) fl_color(this->parent()->labelcolor());
    else fl_color(this->parent()->color());
int GX=16;if (lx<GX*3) GX=-1;
for (i=1;i<GX;i++){
   int tmp=(int)(lx/(REALTYPE)GX*i);
   fl_line(ox+tmp,oy+2,ox+tmp,oy+ly-2);
};
int GY=8;if (ly<GY*3) GY=-1;
for (i=1;i<GY;i++){
   int tmp=(int)(ly/(REALTYPE)GY*i);
   fl_line(ox+2,oy+tmp,ox+lx-2,oy+tmp);
};

//draw the function
fl_line_style(0,1);
fl_line(ox+2,oy+ly/2,ox+lx-2,oy+ly/2);
if (this->active_r()) fl_color(this->parent()->selection_color());
    else fl_color(this->parent()->labelcolor());
int lw=1;
//if ((lx<135)||(ly<135)) lw=1;
fl_line_style(0,lw);
int ph=(int)((phase-64.0)/128.0*OSCIL_SIZE+OSCIL_SIZE);
for (i=1;i<lx;i++){
   int k1=(int)((REALTYPE)OSCIL_SIZE*(i-1)/lx)+ph;
   int k2=(int)((REALTYPE)OSCIL_SIZE*i/lx)+ph;
   REALTYPE y1=smps[k1%OSCIL_SIZE]/max;
   REALTYPE y2=smps[k2%OSCIL_SIZE]/max;
   fl_line(i-1+ox,(int)(y1*ly/2.0)+oy+ly/2,i+ox,(int)(y2*ly/2.0)+oy+ly/2);
};
}

void Oscilharmonic::cb_mag_i(PSlider* o, void*) {
  int x=64;
if (Fl::event_button3()) o->value(x);
   else x=127-(int)o->value();
if (x==64) o->selection_color(0);
    else o->selection_color(222);

pthread_mutex_lock(&master->mutex);
 oscil->Phmag[n]=x;
 if (x==64) {
    oscil->Phphase[n]=64;
    phase->value(64);
 };
 oscil->prepare();
pthread_mutex_unlock(&master->mutex);

display->redraw();
oldosc->redraw();
if (cbwidget!=NULL) {
	cbwidget->do_callback();
	applybutton->color(FL_RED);
	applybutton->redraw();
};
}
void Oscilharmonic::cb_mag(PSlider* o, void* v) {
  ((Oscilharmonic*)(o->parent()->user_data()))->cb_mag_i(o,v);
}

void Oscilharmonic::cb_phase_i(PSlider* o, void*) {
  int x=64;
if (Fl::event_button3()) o->value(x);
   else x=(int)o->value();

pthread_mutex_lock(&master->mutex);
 oscil->Phphase[n]=x;
 oscil->prepare();
pthread_mutex_unlock(&master->mutex);

display->redraw();
oldosc->redraw();
if (cbwidget!=NULL) {
	cbwidget->do_callback();
	applybutton->color(FL_RED);
	applybutton->redraw();
};
}
void Oscilharmonic::cb_phase(PSlider* o, void* v) {
  ((Oscilharmonic*)(o->parent()->user_data()))->cb_phase_i(o,v);
}

Fl_Group* Oscilharmonic::make_window() {
  Fl_Group* w;
  { Fl_Group* o = harmonic = new Fl_Group(0, 0, 90, 225);
    w = o;
    o->box(FL_FLAT_BOX);
    o->color(FL_BACKGROUND_COLOR);
    o->selection_color(FL_BACKGROUND_COLOR);
    o->labeltype(FL_NO_LABEL);
    o->labelfont(0);
    o->labelsize(14);
    o->labelcolor(FL_FOREGROUND_COLOR);
    o->user_data((void*)(this));
    o->align(FL_ALIGN_TOP);
    o->when(FL_WHEN_RELEASE);
    { PSlider* o = mag = new PSlider(0, 15, 15, 115);
      o->type(4);
      o->box(FL_FLAT_BOX);
      o->color(FL_BACKGROUND_COLOR);
      o->selection_color((Fl_Color)222);
      o->labeltype(FL_NORMAL_LABEL);
      o->labelfont(0);
      o->labelsize(14);
      o->labelcolor(FL_FOREGROUND_COLOR);
      o->maximum(127);
      o->step(1);
      o->value(64);
      o->callback((Fl_Callback*)cb_mag);
      o->align(FL_ALIGN_BOTTOM);
      o->when(FL_WHEN_CHANGED);
      o->value(127-oscil->Phmag[n]);
      if (oscil->Phmag[n]==64) o->selection_color(0);
    }
    { PSlider* o = phase = new PSlider(0, 135, 15, 75);
      o->type(4);
      o->box(FL_FLAT_BOX);
      o->color(FL_BACKGROUND_COLOR);
      o->selection_color((Fl_Color)222);
      o->labeltype(FL_NORMAL_LABEL);
      o->labelfont(0);
      o->labelsize(14);
      o->labelcolor(FL_FOREGROUND_COLOR);
      o->maximum(127);
      o->step(1);
      o->value(64);
      o->callback((Fl_Callback*)cb_phase);
      o->align(FL_ALIGN_BOTTOM);
      o->when(FL_WHEN_CHANGED);
      o->value(oscil->Phphase[n]);
    }
    { Fl_Box* o = new Fl_Box(15, 70, 5, 5);
      o->box(FL_FLAT_BOX);
      o->color(FL_DARK2);
    }
    { Fl_Box* o = new Fl_Box(15, 170, 5, 5);
      o->box(FL_FLAT_BOX);
      o->color(FL_DARK2);
    }
    { Fl_Box* o = new Fl_Box(0, 210, 20, 15, "01");
      o->labelfont(1);
      o->labelsize(9);
      o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
      char tmp[10];snprintf(tmp,10,"%d",n+1);o->label(strdup(tmp));
    }
    { Fl_Box* o = new Fl_Box(0, 0, 20, 15, "01");
      o->labelfont(1);
      o->labelsize(9);
      o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
      char tmp[10];snprintf(tmp,10,"%d",n+1);o->label(strdup(tmp));
    }
    o->end();
  }
  return w;
}

Oscilharmonic::Oscilharmonic(int x,int y, int w, int h, const char *label):Fl_Group(x,y,w,h,label) {
  n=0;
oscil=NULL;
display=NULL;
applybutton=NULL;
cbwidget=NULL;
}

void Oscilharmonic::init(OscilGen *oscil_,int n_,Fl_Group *display_,Fl_Widget *oldosc_,Fl_Widget *cbwidget_,Fl_Widget *applybutton_, Master *master_) {
  oscil=oscil_;
n=n_;
display=display_;
master=master_;
oldosc=oldosc_;
cbwidget=cbwidget_;
applybutton=applybutton_;
make_window();
end();
harmonic->show();
}

void Oscilharmonic::refresh() {
  mag->value(127-oscil->Phmag[n]);
phase->value(oscil->Phphase[n]);

if (oscil->Phmag[n]==64) mag->selection_color(0);
    else mag->selection_color(222);
}

Oscilharmonic::~Oscilharmonic() {
  harmonic->hide();
//delete(harmonic);
}

void OscilEditor::cb_applybutton_i(Fl_Button*, void*) {
  applybutton->color(FL_GRAY);
applybutton->redraw();
if (cbapplywidget!=NULL) {
	cbapplywidget->do_callback();
	cbapplywidget->color(FL_GRAY);
	cbapplywidget->redraw();
};
}
void OscilEditor::cb_applybutton(Fl_Button* o, void* v) {
  ((OscilEditor*)(o->parent()->user_data()))->cb_applybutton_i(o,v);
}

void OscilEditor::cb_rndslider_i(Fl_Value_Slider* o, void*) {
  oscil->Prand=(int)o->value()+64;
oscildisplaygroup->redraw();
oldosc->redraw();
}
void OscilEditor::cb_rndslider(Fl_Value_Slider* o, void* v) {
  ((OscilEditor*)(o->parent()->parent()->user_data()))->cb_rndslider_i(o,v);
}

void OscilEditor::cb_hrndtype_i(Fl_Choice* o, void*) {
  oscil->Pamprandtype=(int) o->value();
}
void OscilEditor::cb_hrndtype(Fl_Choice* o, void* v) {
  ((OscilEditor*)(o->parent()->parent()->parent()->user_data()))->cb_hrndtype_i(o,v);
}

Fl_Menu_Item OscilEditor::menu_hrndtype[] = {
 {"None", 0,  0, 0, 0, FL_NORMAL_LABEL, 1, 10, 0},
 {"Pow", 0,  0, 0, 0, FL_NORMAL_LABEL, 1, 10, 0},
 {"Sin", 0,  0, 0, 0, FL_NORMAL_LABEL, 1, 10, 0},
 {0,0,0,0,0,0,0,0,0}
};

void OscilEditor::cb_hrnddial_i(WidgetPDial* o, void*) {
  oscil->Pamprandpower=(int) o->value();
}
void OscilEditor::cb_hrnddial(WidgetPDial* o, void* v) {
  ((OscilEditor*)(o->parent()->parent()->parent()->user_data()))->cb_hrnddial_i(o,v);
}

void OscilEditor::cb_bfslider_i(WidgetPDial* o, void*) {
  oscil->Pbasefuncpar=(int)o->value()+64;
basefuncdisplaygroup->redraw();
bfparval->value(oscil->Pbasefuncpar-64);

redrawoscil();
}
void OscilEditor::cb_bfslider(WidgetPDial* o, void* v) {
  ((OscilEditor*)(o->parent()->parent()->user_data()))->cb_bfslider_i(o,v);
}

void OscilEditor::cb_bftype_i(Fl_Choice* o, void*) {
  oscil->Pcurrentbasefunc=(int) o->value();

basefuncdisplaygroup->redraw();
redrawoscil();

if ((oscil->Pcurrentbasefunc==0)||(oscil->Pcurrentbasefunc==127)) basefuncmodulation->deactivate();
      else basefuncmodulation->activate();
}
void OscilEditor::cb_bftype(Fl_Choice* o, void* v) {
  ((OscilEditor*)(o->parent()->parent()->user_data()))->cb_bftype_i(o,v);
}

Fl_Menu_Item OscilEditor::menu_bftype[] = {
 {"Sine", 0,  0, 0, 0, FL_NORMAL_LABEL, 1, 12, 0},
 {"Triangle", 0,  0, 0, 0, FL_NORMAL_LABEL, 1, 12, 0},
 {"Pulse", 0,  0, 0, 0, FL_NORMAL_LABEL, 1, 12, 0},
 {"Saw", 0,  0, 0, 0, FL_NORMAL_LABEL, 1, 12, 0},
 {"Power", 0,  0, 0, 0, FL_NORMAL_LABEL, 1, 12, 0},
 {"Gauss", 0,  0, 0, 0, FL_NORMAL_LABEL, 1, 12, 0},
 {"Diode", 0,  0, 0, 0, FL_NORMAL_LABEL, 1, 12, 0},
 {"AbsSine", 0,  0, 0, 0, FL_NORMAL_LABEL, 1, 12, 0},
 {"PulseSine", 0,  0, 0, 0, FL_NORMAL_LABEL, 1, 12, 0},
 {"StrchSine", 0,  0, 0, 0, FL_NORMAL_LABEL, 1, 12, 0},
 {"Chirp", 0,  0, 0, 0, FL_NORMAL_LABEL, 1, 12, 0},
 {"AbsStrSine", 0,  0, 0, 0, FL_NORMAL_LABEL, 1, 12, 0},
 {"Chebyshev", 0,  0, 0, 0, FL_NORMAL_LABEL, 1, 12, 0},
 {"Sqr", 0,  0, 0, 0, FL_NORMAL_LABEL, 1, 12, 0},
 {0,0,0,0,0,0,0,0,0}
};

void OscilEditor::cb_bfmodtype_i(Fl_Choice* o, void*) {
  oscil->Pbasefuncmodulation=(int) o->value();
basefuncdisplaygroup->redraw();
redrawoscil();
}
void OscilEditor::cb_bfmodtype(Fl_Choice* o, void* v) {
  ((OscilEditor*)(o->parent()->parent()->parent()->user_data()))->cb_bfmodtype_i(o,v);
}

Fl_Menu_Item OscilEditor::menu_bfmodtype[] = {
 {"None", 0,  0, 0, 0, FL_NORMAL_LABEL, 1, 10, 0},
 {"Rev", 0,  0, 0, 0, FL_NORMAL_LABEL, 1, 10, 0},
 {"Sine", 0,  0, 0, 0, FL_NORMAL_LABEL, 1, 10, 0},
 {"Pow", 0,  0, 0, 0, FL_NORMAL_LABEL, 1, 10, 0},
 {0,0,0,0,0,0,0,0,0}
};

void OscilEditor::cb_bfmodpar1_i(WidgetPDial* o, void*) {
  oscil->Pbasefuncmodulationpar1=(int)o->value();
basefuncdisplaygroup->redraw();
redrawoscil();
}
void OscilEditor::cb_bfmodpar1(WidgetPDial* o, void* v) {
  ((OscilEditor*)(o->parent()->parent()->parent()->user_data()))->cb_bfmodpar1_i(o,v);
}

void OscilEditor::cb_bfmodpar2_i(WidgetPDial* o, void*) {
  oscil->Pbasefuncmodulationpar2=(int)o->value();
basefuncdisplaygroup->redraw();
redrawoscil();
}
void OscilEditor::cb_bfmodpar2(WidgetPDial* o, void* v) {
  ((OscilEditor*)(o->parent()->parent()->parent()->user_data()))->cb_bfmodpar2_i(o,v);
}

void OscilEditor::cb_bfmodpar3_i(WidgetPDial* o, void*) {
  oscil->Pbasefuncmodulationpar3=(int)o->value();
basefuncdisplaygroup->redraw();
redrawoscil();
}
void OscilEditor::cb_bfmodpar3(WidgetPDial* o, void* v) {
  ((OscilEditor*)(o->parent()->parent()->parent()->user_data()))->cb_bfmodpar3_i(o,v);
}

void OscilEditor::cb_magtype_i(Fl_Choice* o, void*) {
  oscil->Phmagtype=(int) o->value();
basefuncdisplaygroup->redraw();

redrawoscil();
}
void OscilEditor::cb_magtype(Fl_Choice* o, void* v) {
  ((OscilEditor*)(o->parent()->user_data()))->cb_magtype_i(o,v);
}

Fl_Menu_Item OscilEditor::menu_magtype[] = {
 {"Linear", 0,  0, 0, 0, FL_NORMAL_LABEL, 1, 12, 0},
 {"-40dB", 0,  0, 0, 0, FL_NORMAL_LABEL, 1, 12, 0},
 {"-60dB", 0,  0, 0, 0, FL_NORMAL_LABEL, 1, 12, 0},
 {"-80dB", 0,  0, 0, 0, FL_NORMAL_LABEL, 1, 12, 0},
 {"-100dB", 0,  0, 0, 0, FL_NORMAL_LABEL, 1, 12, 0},
 {0,0,0,0,0,0,0,0,0}
};

void OscilEditor::cb_Use_i(Fl_Button*, void*) {
  oscil->useasbase();
if (autoclearbutton->value()){
   for (int i=0;i<MAX_AD_HARMONICS;i++){
      h[i]->mag->value(64);
      oscil->Phmag[i]=64;
      h[i]->phase->value(64);
      oscil->Phphase[i]=64;
   };
   oscil->Phmag[0]=127;

   oscil->Pharmonicshift=0;
   harmonicshiftcounter->value(0);

   h[0]->mag->value(0);
   wshbutton->value(0);
   wshbutton->do_callback();
   fltbutton->value(0);
   fltbutton->do_callback();
   sabutton->value(0);
   sabutton->do_callback();
};

pthread_mutex_lock(&master->mutex);
 for (int i=0;i<MAX_AD_HARMONICS;i++){
   if (oscil->Phmag[i]==64) h[i]->mag->selection_color(0);
      else h[i]->mag->selection_color(222);
 };
 oscil->prepare();
pthread_mutex_unlock(&master->mutex);

basefuncdisplaygroup->redraw();
redrawoscil();
}
void OscilEditor::cb_Use(Fl_Button* o, void* v) {
  ((OscilEditor*)(o->parent()->user_data()))->cb_Use_i(o,v);
}

void OscilEditor::cb_Close_i(Fl_Button*, void*) {
  osceditUI->hide();
}
void OscilEditor::cb_Close(Fl_Button* o, void* v) {
  ((OscilEditor*)(o->parent()->user_data()))->cb_Close_i(o,v);
}

void OscilEditor::cb_Clear_i(Fl_Button*, void*) {
  if (!fl_ask("Clear the harmonics settings?")) return;

for (int i=0;i<MAX_AD_HARMONICS;i++){
    h[i]->mag->value(64);
    oscil->Phmag[i]=64;
    h[i]->phase->value(64);
    oscil->Phphase[i]=64;
};
oscil->Phmag[0]=127;
h[0]->mag->value(0);

for (int i=0;i<MAX_AD_HARMONICS;i++){
  if (oscil->Phmag[i]==64) h[i]->mag->selection_color(0);
     else h[i]->mag->selection_color(222);
};

//harmonics->redraw();

pthread_mutex_lock(&master->mutex);
 oscil->prepare();
pthread_mutex_unlock(&master->mutex);

redrawoscil();
}
void OscilEditor::cb_Clear(Fl_Button* o, void* v) {
  ((OscilEditor*)(o->parent()->user_data()))->cb_Clear_i(o,v);
}

void OscilEditor::cb_wshbutton_i(Fl_Choice* o, void*) {
  oscil->Pwaveshapingfunction=(int) o->value();
basefuncdisplaygroup->redraw();
redrawoscil();
}
void OscilEditor::cb_wshbutton(Fl_Choice* o, void* v) {
  ((OscilEditor*)(o->parent()->parent()->user_data()))->cb_wshbutton_i(o,v);
}

Fl_Menu_Item OscilEditor::menu_wshbutton[] = {
 {"None", 0,  0, 0, 0, FL_NORMAL_LABEL, 1, 10, 0},
 {"Atan", 0,  0, 0, 0, FL_NORMAL_LABEL, 1, 10, 0},
 {"Asym1", 0,  0, 0, 0, FL_NORMAL_LABEL, 1, 10, 0},
 {"Pow", 0,  0, 0, 0, FL_NORMAL_LABEL, 1, 10, 0},
 {"Sine", 0,  0, 0, 0, FL_NORMAL_LABEL, 1, 10, 0},
 {"Qnts", 0,  0, 0, 0, FL_NORMAL_LABEL, 1, 10, 0},
 {"Zigzg", 0,  0, 0, 0, FL_NORMAL_LABEL, 1, 10, 0},
 {"Lmt", 0,  0, 0, 0, FL_NORMAL_LABEL, 1, 10, 0},
 {"LmtU", 0,  0, 0, 0, FL_NORMAL_LABEL, 1, 10, 0},
 {"LmtL", 0,  0, 0, 0, FL_NORMAL_LABEL, 1, 10, 0},
 {"ILmt", 0,  0, 0, 0, FL_NORMAL_LABEL, 1, 10, 0},
 {"Clip", 0,  0, 0, 0, FL_NORMAL_LABEL, 1, 10, 0},
 {"Asym2", 0,  0, 0, 0, FL_NORMAL_LABEL, 1, 10, 0},
 {"Pow2", 0,  0, 0, 0, FL_NORMAL_LABEL, 1, 10, 0},
 {"Sgm", 0,  0, 0, 0, FL_NORMAL_LABEL, 1, 10, 0},
 {0,0,0,0,0,0,0,0,0}
};

void OscilEditor::cb_wshpar_i(WidgetPDial* o, void*) {
  oscil->Pwaveshaping=(int)o->value()+64;
wsparval->value(oscil->Pwaveshaping-64);
redrawoscil();
}
void OscilEditor::cb_wshpar(WidgetPDial* o, void* v) {
  ((OscilEditor*)(o->parent()->parent()->user_data()))->cb_wshpar_i(o,v);
}

void OscilEditor::cb_fltbutton_i(Fl_Choice* o, void*) {
  oscil->Pfiltertype=(int) o->value();

redrawoscil();
}
void OscilEditor::cb_fltbutton(Fl_Choice* o, void* v) {
  ((OscilEditor*)(o->parent()->parent()->user_data()))->cb_fltbutton_i(o,v);
}

Fl_Menu_Item OscilEditor::menu_fltbutton[] = {
 {"None", 0,  0, 0, 0, FL_NORMAL_LABEL, 1, 10, 0},
 {"LP1", 0,  0, 0, 0, FL_NORMAL_LABEL, 1, 10, 0},
 {"HP1a", 0,  0, 0, 0, FL_NORMAL_LABEL, 1, 10, 0},
 {"HP1b", 0,  0, 0, 0, FL_NORMAL_LABEL, 1, 10, 0},
 {"BP1", 0,  0, 0, 0, FL_NORMAL_LABEL, 1, 10, 0},
 {"BS1", 0,  0, 0, 0, FL_NORMAL_LABEL, 1, 10, 0},
 {"LP2", 0,  0, 0, 0, FL_NORMAL_LABEL, 1, 10, 0},
 {"HP2", 0,  0, 0, 0, FL_NORMAL_LABEL, 1, 10, 0},
 {"BP2", 0,  0, 0, 0, FL_NORMAL_LABEL, 1, 10, 0},
 {"BS2", 0,  0, 0, 0, FL_NORMAL_LABEL, 1, 10, 0},
 {"Cos", 0,  0, 0, 0, FL_NORMAL_LABEL, 1, 10, 0},
 {"Sin", 0,  0, 0, 0, FL_NORMAL_LABEL, 1, 10, 0},
 {"LSh", 0,  0, 0, 0, FL_NORMAL_LABEL, 1, 10, 0},
 {"S", 0,  0, 0, 0, FL_NORMAL_LABEL, 1, 10, 0},
 {0,0,0,0,0,0,0,0,0}
};

void OscilEditor::cb_filtervalue1_i(WidgetPDial* o, void*) {
  oscil->Pfilterpar1=(int)o->value();

redrawoscil();
}
void OscilEditor::cb_filtervalue1(WidgetPDial* o, void* v) {
  ((OscilEditor*)(o->parent()->parent()->user_data()))->cb_filtervalue1_i(o,v);
}

void OscilEditor::cb_filterpref_i(Fl_Check_Button* o, void*) {
  oscil->Pfilterbeforews=(int)o->value();

redrawoscil();
}
void OscilEditor::cb_filterpref(Fl_Check_Button* o, void* v) {
  ((OscilEditor*)(o->parent()->parent()->user_data()))->cb_filterpref_i(o,v);
}

void OscilEditor::cb_filtervalue2_i(WidgetPDial* o, void*) {
  oscil->Pfilterpar2=(int)o->value();

redrawoscil();
}
void OscilEditor::cb_filtervalue2(WidgetPDial* o, void* v) {
  ((OscilEditor*)(o->parent()->parent()->user_data()))->cb_filtervalue2_i(o,v);
}

void OscilEditor::cb_sabutton_i(Fl_Choice* o, void*) {
  oscil->Psatype=(int) o->value();
redrawoscil();
}
void OscilEditor::cb_sabutton(Fl_Choice* o, void* v) {
  ((OscilEditor*)(o->parent()->parent()->user_data()))->cb_sabutton_i(o,v);
}

Fl_Menu_Item OscilEditor::menu_sabutton[] = {
 {"None", 0,  0, 0, 0, FL_NORMAL_LABEL, 1, 10, 0},
 {"Pow", 0,  0, 0, 0, FL_NORMAL_LABEL, 1, 10, 0},
 {"ThrsD", 0,  0, 0, 0, FL_NORMAL_LABEL, 1, 10, 0},
 {"ThrsU", 0,  0, 0, 0, FL_NORMAL_LABEL, 1, 10, 0},
 {0,0,0,0,0,0,0,0,0}
};

void OscilEditor::cb_sadjpar_i(WidgetPDial* o, void*) {
  oscil->Psapar=(int)o->value();
redrawoscil();
}
void OscilEditor::cb_sadjpar(WidgetPDial* o, void* v) {
  ((OscilEditor*)(o->parent()->parent()->user_data()))->cb_sadjpar_i(o,v);
}

void OscilEditor::cb_harmonicshiftcounter_i(Fl_Counter* o, void*) {
  oscil->Pharmonicshift=(int)o->value();
redrawoscil();
}
void OscilEditor::cb_harmonicshiftcounter(Fl_Counter* o, void* v) {
  ((OscilEditor*)(o->parent()->parent()->user_data()))->cb_harmonicshiftcounter_i(o,v);
}

void OscilEditor::cb_harmonicshiftpre_i(Fl_Check_Button* o, void*) {
  oscil->Pharmonicshiftfirst=(int)o->value();
redrawoscil();
}
void OscilEditor::cb_harmonicshiftpre(Fl_Check_Button* o, void* v) {
  ((OscilEditor*)(o->parent()->parent()->user_data()))->cb_harmonicshiftpre_i(o,v);
}

void OscilEditor::cb_R_i(Fl_Button*, void*) {
  oscil->Pharmonicshift=0;
harmonicshiftcounter->value(0);
redrawoscil();
}
void OscilEditor::cb_R(Fl_Button* o, void* v) {
  ((OscilEditor*)(o->parent()->parent()->user_data()))->cb_R_i(o,v);
}

void OscilEditor::cb_adhrtype_i(Fl_Choice* o, void*) {
  oscil->Padaptiveharmonics=(int) o->value();
redrawoscil();
}
void OscilEditor::cb_adhrtype(Fl_Choice* o, void* v) {
  ((OscilEditor*)(o->parent()->parent()->user_data()))->cb_adhrtype_i(o,v);
}

Fl_Menu_Item OscilEditor::menu_adhrtype[] = {
 {"OFF", 0,  0, 0, 0, FL_NORMAL_LABEL, 1, 10, 0},
 {"ON", 0,  0, 0, 0, FL_NORMAL_LABEL, 1, 10, 0},
 {"Square", 0,  0, 0, 0, FL_NORMAL_LABEL, 1, 10, 0},
 {"2xSub", 0,  0, 0, 0, FL_NORMAL_LABEL, 1, 10, 0},
 {"2xAdd", 0,  0, 0, 0, FL_NORMAL_LABEL, 1, 10, 0},
 {"3xSub", 0,  0, 0, 0, FL_NORMAL_LABEL, 1, 10, 0},
 {"3xAdd", 0,  0, 0, 0, FL_NORMAL_LABEL, 1, 10, 0},
 {"4xSub", 0,  0, 0, 0, FL_NORMAL_LABEL, 1, 10, 0},
 {"4xAdd", 0,  0, 0, 0, FL_NORMAL_LABEL, 1, 10, 0},
 {0,0,0,0,0,0,0,0,0}
};

void OscilEditor::cb_adhrpow_i(WidgetPDial* o, void*) {
  oscil->Padaptiveharmonicspower=(int)o->value();
redrawoscil();
}
void OscilEditor::cb_adhrpow(WidgetPDial* o, void* v) {
  ((OscilEditor*)(o->parent()->parent()->user_data()))->cb_adhrpow_i(o,v);
}

void OscilEditor::cb_adhrbf_i(WidgetPDial* o, void*) {
  oscil->Padaptiveharmonicsbasefreq=(int)o->value();
redrawoscil();
}
void OscilEditor::cb_adhrbf(WidgetPDial* o, void* v) {
  ((OscilEditor*)(o->parent()->parent()->user_data()))->cb_adhrbf_i(o,v);
}

void OscilEditor::cb_adhrpar_i(Fl_Slider* o, void*) {
  oscil->Padaptiveharmonicspar=(int)o->value();
redrawoscil();
}
void OscilEditor::cb_adhrpar(Fl_Slider* o, void* v) {
  ((OscilEditor*)(o->parent()->parent()->user_data()))->cb_adhrpar_i(o,v);
}

void OscilEditor::cb_modtype_i(Fl_Choice* o, void*) {
  oscil->Pmodulation=(int) o->value();

redrawoscil();
}
void OscilEditor::cb_modtype(Fl_Choice* o, void* v) {
  ((OscilEditor*)(o->parent()->parent()->user_data()))->cb_modtype_i(o,v);
}

Fl_Menu_Item OscilEditor::menu_modtype[] = {
 {"None", 0,  0, 0, 0, FL_NORMAL_LABEL, 1, 10, 0},
 {"Rev", 0,  0, 0, 0, FL_NORMAL_LABEL, 1, 10, 0},
 {"Sine", 0,  0, 0, 0, FL_NORMAL_LABEL, 1, 10, 0},
 {"Pow", 0,  0, 0, 0, FL_NORMAL_LABEL, 1, 10, 0},
 {0,0,0,0,0,0,0,0,0}
};

void OscilEditor::cb_modpar1_i(WidgetPDial* o, void*) {
  oscil->Pmodulationpar1=(int)o->value();

redrawoscil();
}
void OscilEditor::cb_modpar1(WidgetPDial* o, void* v) {
  ((OscilEditor*)(o->parent()->parent()->user_data()))->cb_modpar1_i(o,v);
}

void OscilEditor::cb_modpar2_i(WidgetPDial* o, void*) {
  oscil->Pmodulationpar2=(int)o->value();

redrawoscil();
}
void OscilEditor::cb_modpar2(WidgetPDial* o, void* v) {
  ((OscilEditor*)(o->parent()->parent()->user_data()))->cb_modpar2_i(o,v);
}

void OscilEditor::cb_modpar3_i(WidgetPDial* o, void*) {
  oscil->Pmodulationpar3=(int)o->value();
redrawoscil();
}
void OscilEditor::cb_modpar3(WidgetPDial* o, void* v) {
  ((OscilEditor*)(o->parent()->parent()->user_data()))->cb_modpar3_i(o,v);
}

void OscilEditor::cb_Sine_i(Fl_Button*, void*) {
  if (!fl_ask("Convert to SINE?")) return;

pthread_mutex_lock(&master->mutex);
 oscil->convert2sine(0);
pthread_mutex_unlock(&master->mutex);

redrawoscil();
refresh();
}
void OscilEditor::cb_Sine(Fl_Button* o, void* v) {
  ((OscilEditor*)(o->parent()->user_data()))->cb_Sine_i(o,v);
}

void OscilEditor::cb_C_i(Fl_Button*, void*) {
  presetsui->copy(oscil);
}
void OscilEditor::cb_C(Fl_Button* o, void* v) {
  ((OscilEditor*)(o->parent()->user_data()))->cb_C_i(o,v);
}

void OscilEditor::cb_P_i(Fl_Button*, void*) {
  presetsui->paste(oscil,this);
}
void OscilEditor::cb_P(Fl_Button* o, void* v) {
  ((OscilEditor*)(o->parent()->user_data()))->cb_P_i(o,v);
}

Fl_Double_Window* OscilEditor::make_window() {
  Fl_Double_Window* w;
  { Fl_Double_Window* o = osceditUI = new Fl_Double_Window(735, 595, "ADsynth Oscillator Editor");
    w = o;
    o->user_data((void*)(this));
    { Fl_Button* o = applybutton = new Fl_Button(300, 280, 60, 20, "Apply");
      o->box(FL_THIN_UP_BOX);
      o->labelfont(1);
      o->callback((Fl_Callback*)cb_applybutton);
      if (!oscil->ADvsPAD) o->hide();
    }
    { Fl_Group* o = oscildisplaygroup = new Fl_Group(5, 5, 360, 300);
      o->box(FL_ENGRAVED_FRAME);
      { Fl_Group* o = new Fl_Group(10, 85, 350, 190);
        o->box(FL_THIN_DOWN_BOX);
        o->color(FL_GRAY0);
        o->selection_color((Fl_Color)71);
        o->labelcolor((Fl_Color)179);
        Oscilloscope *osc=new Oscilloscope(o->x(),o->y(),o->w(),o->h(),"");
        osc->init(oscil,master);
        o->end();
      }
      { Fl_Box* o = new Fl_Box(120, 10, 110, 20, "Oscillator");
        o->box(FL_FLAT_BOX);
        o->labelfont(1);
      }
      { Fl_Value_Slider* o = rndslider = new Fl_Value_Slider(140, 285, 100, 10, "rnd");
        o->tooltip("Oscilator Phase Randomness: smaller than 0 is \"group\", larger than 0 is for\
 each harmonic");
        o->type(5);
        o->box(FL_FLAT_BOX);
        o->labelsize(10);
        o->minimum(-64);
        o->maximum(63);
        o->step(1);
        o->callback((Fl_Callback*)cb_rndslider);
        o->align(FL_ALIGN_TOP_LEFT);
        if (oscil->ADvsPAD) o->hide();
      }
      { Fl_Group* o = new Fl_Group(10, 30, 350, 50);
        o->box(FL_THIN_DOWN_BOX);
        o->color(FL_GRAY0);
        o->selection_color((Fl_Color)218);
        o->labelcolor(FL_GREEN);
        OscilSpectrum *spc=new OscilSpectrum(o->x(),o->y(),o->w(),o->h(),"");
        spc->init(oscil,0,master);
        o->end();
      }
      { Fl_Group* o = new Fl_Group(246, 277, 115, 25);
        o->box(FL_ENGRAVED_BOX);
        { Fl_Choice* o = hrndtype = new Fl_Choice(281, 282, 50, 15, "H.rnd");
          o->tooltip("Harmonic Amplitude Randomness");
          o->down_box(FL_BORDER_BOX);
          o->labelsize(10);
          o->textsize(10);
          o->callback((Fl_Callback*)cb_hrndtype);
          o->menu(menu_hrndtype);
        }
        { WidgetPDial* o = hrnddial = new WidgetPDial(338, 280, 18, 18);
          o->tooltip("Oscillator\'s spectrum adjust parameter");
          o->box(FL_OVAL_BOX);
          o->color(FL_BACKGROUND_COLOR);
          o->selection_color(FL_INACTIVE_COLOR);
          o->labeltype(FL_NORMAL_LABEL);
          o->labelfont(0);
          o->labelsize(14);
          o->labelcolor(FL_FOREGROUND_COLOR);
          o->maximum(127);
          o->step(1);
          o->callback((Fl_Callback*)cb_hrnddial);
          o->align(FL_ALIGN_BOTTOM);
          o->when(FL_WHEN_CHANGED);
        }
        if (oscil->ADvsPAD) o->hide();
        o->end();
      }
      o->end();
    }
    { Fl_Box* o = new Fl_Box(495, 15, 110, 20, "Base Func.");
      o->box(FL_FLAT_BOX);
      o->labelfont(1);
    }
    { Fl_Group* o = basefuncdisplaygroup = new Fl_Group(365, 5, 360, 300);
      o->box(FL_ENGRAVED_FRAME);
      { Fl_Group* o = new Fl_Group(370, 85, 350, 190);
        o->box(FL_THIN_DOWN_BOX);
        o->color(FL_GRAY0);
        o->selection_color((Fl_Color)71);
        o->labelcolor((Fl_Color)179);
        Oscilloscope *osc=new Oscilloscope(o->x(),o->y(),o->w(),o->h(),"");
        osc->init(oscil,1,master);
        o->end();
      }
      { WidgetPDial* o = bfslider = new WidgetPDial(520, 280, 20, 20);
        o->tooltip("Base Function Parameter");
        o->box(FL_OVAL_BOX);
        o->color(FL_BACKGROUND_COLOR);
        o->selection_color(FL_INACTIVE_COLOR);
        o->labeltype(FL_NORMAL_LABEL);
        o->labelfont(0);
        o->labelsize(14);
        o->labelcolor(FL_FOREGROUND_COLOR);
        o->minimum(-64);
        o->maximum(63);
        o->step(1);
        o->callback((Fl_Callback*)cb_bfslider);
        o->align(FL_ALIGN_BOTTOM);
        o->when(FL_WHEN_CHANGED);
      }
      { Fl_Choice* o = bftype = new Fl_Choice(370, 285, 90, 15, "Base.F..");
        o->down_box(FL_BORDER_BOX);
        o->labelsize(10);
        o->textsize(12);
        o->callback((Fl_Callback*)cb_bftype);
        o->align(FL_ALIGN_TOP_LEFT);
        o->menu(menu_bftype);
      }
      { Fl_Box* o = new Fl_Box(480, 10, 110, 20, "Base Func.");
        o->box(FL_FLAT_BOX);
        o->labelfont(1);
      }
      { Fl_Group* o = new Fl_Group(370, 30, 350, 50);
        o->box(FL_THIN_DOWN_BOX);
        o->color(FL_GRAY0);
        o->selection_color((Fl_Color)218);
        o->labelcolor(FL_GREEN);
        OscilSpectrum *spc=new OscilSpectrum (o->x(),o->y(),o->w(),o->h(),"");
        spc->init(oscil,1,master);
        o->end();
      }
      { Fl_Value_Output* o = bfparval = new Fl_Value_Output(490, 285, 25, 15, "Par.");
        o->labelsize(13);
        o->minimum(-63);
        o->maximum(63);
        o->step(1);
      }
      { Fl_Group* o = basefuncmodulation = new Fl_Group(550, 276, 169, 25);
        o->box(FL_ENGRAVED_BOX);
        { Fl_Choice* o = bfmodtype = new Fl_Choice(599, 281, 50, 15, "B.F.Mod.");
          o->tooltip("Base function modulation");
          o->down_box(FL_BORDER_BOX);
          o->labelsize(10);
          o->textsize(10);
          o->callback((Fl_Callback*)cb_bfmodtype);
          o->menu(menu_bfmodtype);
        }
        { WidgetPDial* o = bfmodpar1 = new WidgetPDial(659, 281, 15, 15);
          o->tooltip("Oscillator\'s modulation parameter 1");
          o->box(FL_OVAL_BOX);
          o->color(FL_BACKGROUND_COLOR);
          o->selection_color(FL_INACTIVE_COLOR);
          o->labeltype(FL_NORMAL_LABEL);
          o->labelfont(0);
          o->labelsize(14);
          o->labelcolor(FL_FOREGROUND_COLOR);
          o->maximum(127);
          o->step(1);
          o->callback((Fl_Callback*)cb_bfmodpar1);
          o->align(FL_ALIGN_BOTTOM);
          o->when(FL_WHEN_CHANGED);
        }
        { WidgetPDial* o = bfmodpar2 = new WidgetPDial(679, 281, 15, 15);
          o->tooltip("Oscillator\'s modulation parameter 2");
          o->box(FL_OVAL_BOX);
          o->color(FL_BACKGROUND_COLOR);
          o->selection_color(FL_INACTIVE_COLOR);
          o->labeltype(FL_NORMAL_LABEL);
          o->labelfont(0);
          o->labelsize(14);
          o->labelcolor(FL_FOREGROUND_COLOR);
          o->maximum(127);
          o->step(1);
          o->callback((Fl_Callback*)cb_bfmodpar2);
          o->align(FL_ALIGN_BOTTOM);
          o->when(FL_WHEN_CHANGED);
        }
        { WidgetPDial* o = bfmodpar3 = new WidgetPDial(699, 281, 15, 15);
          o->tooltip("Oscillator\'s modulation parameter 3");
          o->box(FL_OVAL_BOX);
          o->color(FL_BACKGROUND_COLOR);
          o->selection_color(FL_INACTIVE_COLOR);
          o->labeltype(FL_NORMAL_LABEL);
          o->labelfont(0);
          o->labelsize(14);
          o->labelcolor(FL_FOREGROUND_COLOR);
          o->maximum(127);
          o->step(1);
          o->callback((Fl_Callback*)cb_bfmodpar3);
          o->align(FL_ALIGN_BOTTOM);
          o->when(FL_WHEN_CHANGED);
        }
        if ((oscil->Pcurrentbasefunc==0)||(oscil->Pcurrentbasefunc==127)) basefuncmodulation->deactivate();
        o->end();
      }
      o->end();
    }
    { Fl_Choice* o = magtype = new Fl_Choice(70, 280, 65, 20, "Mag.Type");
      o->down_box(FL_BORDER_BOX);
      o->labelsize(12);
      o->textsize(12);
      o->callback((Fl_Callback*)cb_magtype);
      o->menu(menu_magtype);
    }
    { Fl_Button* o = new Fl_Button(5, 313, 85, 20, "Use as base");
      o->tooltip("Use this Oscillator as base function");
      o->box(FL_THIN_UP_BOX);
      o->labelfont(1);
      o->labelsize(12);
      o->callback((Fl_Callback*)cb_Use);
    }
    { Fl_Button* o = new Fl_Button(668, 565, 62, 25, "Close");
      o->box(FL_THIN_UP_BOX);
      o->callback((Fl_Callback*)cb_Close);
    }
    { Fl_Button* o = new Fl_Button(670, 505, 55, 15, "Clear");
      o->box(FL_THIN_UP_BOX);
      o->labelfont(1);
      o->labelsize(12);
      o->callback((Fl_Callback*)cb_Clear);
    }
    { Fl_Group* o = new Fl_Group(135, 308, 150, 30);
      o->box(FL_ENGRAVED_BOX);
      { Fl_Choice* o = wshbutton = new Fl_Choice(165, 313, 55, 20, "Wsh.");
        o->tooltip("Waveshaping function");
        o->down_box(FL_BORDER_BOX);
        o->labelsize(10);
        o->textsize(10);
        o->callback((Fl_Callback*)cb_wshbutton);
        o->menu(menu_wshbutton);
      }
      { WidgetPDial* o = wshpar = new WidgetPDial(260, 313, 20, 20);
        o->tooltip("Waveshaping Parameter");
        o->box(FL_OVAL_BOX);
        o->color(FL_BACKGROUND_COLOR);
        o->selection_color(FL_INACTIVE_COLOR);
        o->labeltype(FL_NORMAL_LABEL);
        o->labelfont(0);
        o->labelsize(14);
        o->labelcolor(FL_FOREGROUND_COLOR);
        o->minimum(-64);
        o->maximum(63);
        o->step(1);
        o->callback((Fl_Callback*)cb_wshpar);
        o->align(FL_ALIGN_BOTTOM);
        o->when(FL_WHEN_CHANGED);
      }
      { Fl_Value_Output* o = wsparval = new Fl_Value_Output(228, 316, 25, 15);
        o->labelsize(13);
        o->minimum(-63);
        o->maximum(63);
        o->step(1);
      }
      o->end();
    }
    { Fl_Light_Button* o = autoclearbutton = new Fl_Light_Button(95, 313, 35, 20, "Clr.");
      o->tooltip("Auto clear when using the oscillator as base function");
      o->box(FL_THIN_UP_BOX);
      o->value(1);
      o->labelfont(1);
      o->labelsize(10);
    }
    { Fl_Group* o = new Fl_Group(285, 308, 155, 30);
      o->box(FL_ENGRAVED_BOX);
      { Fl_Choice* o = fltbutton = new Fl_Choice(315, 313, 50, 20, "Filter");
        o->tooltip("Oscillator\'s filter type");
        o->down_box(FL_BORDER_BOX);
        o->labelsize(10);
        o->textsize(10);
        o->callback((Fl_Callback*)cb_fltbutton);
        o->menu(menu_fltbutton);
      }
      { WidgetPDial* o = filtervalue1 = new WidgetPDial(367, 313, 20, 20);
        o->tooltip("Oscillator\'s filter parameter1");
        o->box(FL_OVAL_BOX);
        o->color(FL_BACKGROUND_COLOR);
        o->selection_color(FL_INACTIVE_COLOR);
        o->labeltype(FL_NORMAL_LABEL);
        o->labelfont(0);
        o->labelsize(14);
        o->labelcolor(FL_FOREGROUND_COLOR);
        o->maximum(127);
        o->step(1);
        o->callback((Fl_Callback*)cb_filtervalue1);
        o->align(FL_ALIGN_BOTTOM);
        o->when(FL_WHEN_CHANGED);
      }
      { Fl_Check_Button* o = filterpref = new Fl_Check_Button(415, 313, 20, 20, "p");
        o->tooltip("Apply the filter before the waveshaping");
        o->down_box(FL_DOWN_BOX);
        o->labelsize(10);
        o->callback((Fl_Callback*)cb_filterpref);
        o->align(FL_ALIGN_RIGHT|FL_ALIGN_INSIDE);
      }
      { WidgetPDial* o = filtervalue2 = new WidgetPDial(392, 313, 20, 20);
        o->tooltip("Oscillator\'s filter parameter2");
        o->box(FL_OVAL_BOX);
        o->color(FL_BACKGROUND_COLOR);
        o->selection_color(FL_INACTIVE_COLOR);
        o->labeltype(FL_NORMAL_LABEL);
        o->labelfont(0);
        o->labelsize(14);
        o->labelcolor(FL_FOREGROUND_COLOR);
        o->maximum(127);
        o->step(1);
        o->callback((Fl_Callback*)cb_filtervalue2);
        o->align(FL_ALIGN_BOTTOM);
        o->when(FL_WHEN_CHANGED);
      }
      o->end();
    }
    { Fl_Group* o = new Fl_Group(590, 308, 135, 30);
      o->box(FL_ENGRAVED_BOX);
      { Fl_Choice* o = sabutton = new Fl_Choice(630, 313, 60, 20, "Sp.adj.");
        o->tooltip("Oscillator\'s spectrum adjust");
        o->down_box(FL_BORDER_BOX);
        o->labelsize(10);
        o->textsize(10);
        o->callback((Fl_Callback*)cb_sabutton);
        o->menu(menu_sabutton);
      }
      { WidgetPDial* o = sadjpar = new WidgetPDial(695, 313, 20, 20);
        o->tooltip("Oscillator\'s spectrum adjust parameter");
        o->box(FL_OVAL_BOX);
        o->color(FL_BACKGROUND_COLOR);
        o->selection_color(FL_INACTIVE_COLOR);
        o->labeltype(FL_NORMAL_LABEL);
        o->labelfont(0);
        o->labelsize(14);
        o->labelcolor(FL_FOREGROUND_COLOR);
        o->maximum(127);
        o->step(1);
        o->callback((Fl_Callback*)cb_sadjpar);
        o->align(FL_ALIGN_BOTTOM);
        o->when(FL_WHEN_CHANGED);
      }
      o->end();
    }
    { Fl_Group* o = new Fl_Group(665, 340, 65, 65);
      o->box(FL_ENGRAVED_BOX);
      { Fl_Counter* o = harmonicshiftcounter = new Fl_Counter(670, 365, 55, 15, "Harmonic Shift");
        o->type(1);
        o->labelsize(10);
        o->minimum(-64);
        o->maximum(64);
        o->step(1);
        o->textfont(1);
        o->textsize(10);
        o->callback((Fl_Callback*)cb_harmonicshiftcounter);
        o->align(129);
      }
      { Fl_Check_Button* o = harmonicshiftpre = new Fl_Check_Button(690, 385, 34, 15, "preH");
        o->tooltip("Apply the harmonic shift before the waveshaping and filtering");
        o->down_box(FL_DOWN_BOX);
        o->labelsize(10);
        o->callback((Fl_Callback*)cb_harmonicshiftpre);
        o->align(FL_ALIGN_RIGHT|FL_ALIGN_INSIDE);
      }
      { Fl_Button* o = new Fl_Button(670, 385, 20, 15, "R");
        o->box(FL_THIN_UP_BOX);
        o->labelfont(1);
        o->labelsize(10);
        o->callback((Fl_Callback*)cb_R);
      }
      o->end();
    }
    { Fl_Group* o = new Fl_Group(665, 410, 65, 90);
      o->box(FL_ENGRAVED_FRAME);
      { Fl_Choice* o = adhrtype = new Fl_Choice(670, 425, 55, 15, "Adpt.Harm.");
        o->tooltip("The type of the addaptive harmonics");
        o->down_box(FL_BORDER_BOX);
        o->labelsize(10);
        o->textsize(10);
        o->callback((Fl_Callback*)cb_adhrtype);
        o->align(129);
        o->when(FL_WHEN_RELEASE_ALWAYS);
        o->menu(menu_adhrtype);
      }
      { WidgetPDial* o = adhrpow = new WidgetPDial(700, 460, 25, 25, "pow");
        o->tooltip("Adaptive harmonics power");
        o->box(FL_OVAL_BOX);
        o->color(FL_BACKGROUND_COLOR);
        o->selection_color(FL_INACTIVE_COLOR);
        o->labeltype(FL_NORMAL_LABEL);
        o->labelfont(0);
        o->labelsize(10);
        o->labelcolor(FL_FOREGROUND_COLOR);
        o->maximum(200);
        o->step(1);
        o->callback((Fl_Callback*)cb_adhrpow);
        o->align(FL_ALIGN_BOTTOM);
        o->when(FL_WHEN_CHANGED);
      }
      { WidgetPDial* o = adhrbf = new WidgetPDial(670, 460, 25, 25, "baseF");
        o->tooltip("Adaptive harmonics base frequency");
        o->box(FL_OVAL_BOX);
        o->color(FL_BACKGROUND_COLOR);
        o->selection_color(FL_INACTIVE_COLOR);
        o->labeltype(FL_NORMAL_LABEL);
        o->labelfont(0);
        o->labelsize(10);
        o->labelcolor(FL_FOREGROUND_COLOR);
        o->maximum(255);
        o->step(1);
        o->callback((Fl_Callback*)cb_adhrbf);
        o->align(FL_ALIGN_BOTTOM);
        o->when(FL_WHEN_CHANGED);
      }
      { Fl_Slider* o = adhrpar = new Fl_Slider(670, 445, 55, 10);
        o->type(5);
        o->box(FL_FLAT_BOX);
        o->maximum(100);
        o->step(1);
        o->value(50);
        o->callback((Fl_Callback*)cb_adhrpar);
      }
      o->end();
    }
    { Fl_Group* o = new Fl_Group(440, 308, 150, 30);
      o->box(FL_ENGRAVED_BOX);
      { Fl_Choice* o = modtype = new Fl_Choice(470, 315, 50, 15, "Mod.");
        o->tooltip("modulation");
        o->down_box(FL_BORDER_BOX);
        o->labelsize(10);
        o->textsize(10);
        o->callback((Fl_Callback*)cb_modtype);
        o->menu(menu_modtype);
      }
      { WidgetPDial* o = modpar1 = new WidgetPDial(530, 315, 15, 15);
        o->tooltip("Oscillator\'s modulation parameter 1");
        o->box(FL_OVAL_BOX);
        o->color(FL_BACKGROUND_COLOR);
        o->selection_color(FL_INACTIVE_COLOR);
        o->labeltype(FL_NORMAL_LABEL);
        o->labelfont(0);
        o->labelsize(14);
        o->labelcolor(FL_FOREGROUND_COLOR);
        o->maximum(127);
        o->step(1);
        o->callback((Fl_Callback*)cb_modpar1);
        o->align(FL_ALIGN_BOTTOM);
        o->when(FL_WHEN_CHANGED);
      }
      { WidgetPDial* o = modpar2 = new WidgetPDial(550, 315, 15, 15);
        o->tooltip("Oscillator\'s modulation parameter 2");
        o->box(FL_OVAL_BOX);
        o->color(FL_BACKGROUND_COLOR);
        o->selection_color(FL_INACTIVE_COLOR);
        o->labeltype(FL_NORMAL_LABEL);
        o->labelfont(0);
        o->labelsize(14);
        o->labelcolor(FL_FOREGROUND_COLOR);
        o->maximum(127);
        o->step(1);
        o->callback((Fl_Callback*)cb_modpar2);
        o->align(FL_ALIGN_BOTTOM);
        o->when(FL_WHEN_CHANGED);
      }
      { WidgetPDial* o = modpar3 = new WidgetPDial(570, 315, 15, 15);
        o->tooltip("Oscillator\'s modulation parameter 3");
        o->box(FL_OVAL_BOX);
        o->color(FL_BACKGROUND_COLOR);
        o->selection_color(FL_INACTIVE_COLOR);
        o->labeltype(FL_NORMAL_LABEL);
        o->labelfont(0);
        o->labelsize(14);
        o->labelcolor(FL_FOREGROUND_COLOR);
        o->maximum(127);
        o->step(1);
        o->callback((Fl_Callback*)cb_modpar3);
        o->align(FL_ALIGN_BOTTOM);
        o->when(FL_WHEN_CHANGED);
      }
      o->end();
    }
    { Fl_Button* o = new Fl_Button(670, 525, 55, 15, "Sine");
      o->box(FL_THIN_UP_BOX);
      o->labelfont(1);
      o->labelsize(12);
      o->callback((Fl_Callback*)cb_Sine);
    }
    { Fl_Button* o = new Fl_Button(670, 545, 25, 15, "C");
      o->box(FL_THIN_UP_BOX);
      o->color((Fl_Color)179);
      o->labelfont(1);
      o->labelsize(12);
      o->labelcolor(FL_BACKGROUND2_COLOR);
      o->callback((Fl_Callback*)cb_C);
    }
    { Fl_Button* o = new Fl_Button(700, 545, 25, 15, "P");
      o->box(FL_THIN_UP_BOX);
      o->color((Fl_Color)179);
      o->labelfont(1);
      o->labelsize(12);
      o->labelcolor(FL_BACKGROUND2_COLOR);
      o->callback((Fl_Callback*)cb_P);
    }
    { Fl_Scroll* o = _this_has_to_be_the_last = new Fl_Scroll(5, 340, 660, 250);
      o->type(1);
      o->box(FL_ENGRAVED_BOX);
      { Fl_Pack* o = harmonics = new Fl_Pack(10, 345, 650, 225);
        o->type(1);
        for (int i=0;i<MAX_AD_HARMONICS;i++){h[i]=new Oscilharmonic(0,0,20,o->h(),"");h[i]->init(oscil,i,oscildisplaygroup,oldosc,cbwidget,applybutton,master);}
        o->end();
      }
      o->end();
    }
    if (oscil->ADvsPAD) o->label("PADsynth Harmonic Content Editor");
    o->end();
  }
  return w;
}

OscilEditor::OscilEditor(OscilGen *oscil_,Fl_Widget *oldosc_,Fl_Widget *cbwidget_,Fl_Widget *cbapplywidget_,Master *master_) {
  oscil=oscil_;
oldosc=oldosc_;
cbwidget=cbwidget_;
cbapplywidget=cbapplywidget_;
master=master_;

make_window();

refresh();
osceditUI->show();
}

OscilEditor::~OscilEditor() {
  osceditUI->hide();
//for (int i=0;i<MAX_AD_HARMONICS;i++) delete (h[i]);
delete (osceditUI);
}

void OscilEditor::refresh() {
  magtype->value(oscil->Phmagtype);
rndslider->value(oscil->Prand-64);

hrndtype->value(oscil->Pamprandtype);
hrnddial->value(oscil->Pamprandpower);

bftype->value(oscil->Pcurrentbasefunc);
bfparval->value(oscil->Pbasefuncpar-64);
bfslider->value(oscil->Pbasefuncpar-64);

bfmodtype->value(oscil->Pbasefuncmodulation);
bfmodpar1->value(oscil->Pbasefuncmodulationpar1);
bfmodpar2->value(oscil->Pbasefuncmodulationpar2);
bfmodpar3->value(oscil->Pbasefuncmodulationpar3);

wshbutton->value(oscil->Pwaveshapingfunction);
wsparval->value(oscil->Pwaveshaping-64);
wshpar->value(oscil->Pwaveshaping-64);

fltbutton->value(oscil->Pfiltertype);
filtervalue1->value(oscil->Pfilterpar1);
filtervalue2->value(oscil->Pfilterpar2);
filterpref->value(oscil->Pfilterbeforews);

modtype->value(oscil->Pmodulation);
modpar1->value(oscil->Pmodulationpar1);
modpar2->value(oscil->Pmodulationpar2);
modpar3->value(oscil->Pmodulationpar3);

sabutton->value(oscil->Psatype);
sadjpar->value(oscil->Psapar);

harmonicshiftcounter->value(oscil->Pharmonicshift);
harmonicshiftpre->value(oscil->Pharmonicshiftfirst);

adhrtype->value(oscil->Padaptiveharmonics);
adhrbf->value(oscil->Padaptiveharmonicsbasefreq);
adhrpow->value(oscil->Padaptiveharmonicspower);
adhrtype->value(oscil->Padaptiveharmonicspar);

for (int i=0;i<MAX_AD_HARMONICS;i++) h[i]->refresh();

pthread_mutex_lock(&master->mutex);
 oscil->prepare();
pthread_mutex_unlock(&master->mutex);

basefuncdisplaygroup->redraw();
redrawoscil();
}

void OscilEditor::redrawoscil() {
  oscildisplaygroup->redraw();
oldosc->redraw();
if (cbwidget!=NULL) {
      cbwidget->do_callback();
      applybutton->color(FL_RED);
      applybutton->redraw();
};
}
